@@ -1,5 +1,8 @@
 // files.c : interactively opening files
 //           wolf 12/94
+
+#pragma implementation
+
 #include "window.h"
 #include "files.h"
 #include <dirent.h>
@@ -136,29 +139,25 @@ void file_display::set_val(char *x) { val = x; redraw(); }
 // 2 callbacks : enter_cb on enter, and act_cb on bpress
 class select_item : public plate {
   VPCP ecb, acb;
-  void *wptr;
-
+  void *wptr; 
 public: 
   char * value;
   select_item(window &parent, VPCP enter_cb, VPCP act_cb, void *wp,
 	      int w, int h, int x, int y) :
     plate(parent,w,h,x,y,flat3d) { 
-    static char empty = 0; value = &empty;
+    static char *empty = ""; value = empty;
     ecb = enter_cb; acb = act_cb, wptr = wp;
     selection_mask |= ButtonPressMask | ButtonReleaseMask;
   }
-
   virtual void redraw() { 
     if (!hidden) { 
       plate::redraw(); 
       if (value) PlaceText(value); else error("select_item == 0"); 
     } 
   }
-  virtual void Enter_CB(XCrossingEvent) {  frame3d(up3d); ecb(wptr,value); }
-  
+  virtual void Enter_CB(XCrossingEvent) { frame3d(up3d); ecb(wptr,value); }
   virtual void Leave_CB(XCrossingEvent) { default_frame(); }
-  virtual void BPress_CB(XButtonEvent) { frame3d(down3d); acb(wptr,value); }
- 
+  virtual void BPress_CB(XButtonEvent) { frame3d(down3d); (*acb)(wptr,value);  } 
   virtual void BRelease_CB(XButtonEvent) { default_frame(); }
   void enable(char *x) { value = x; hidden = False; redraw(); }
   void disable() { hidden = True; }
@@ -341,7 +340,7 @@ int filter(const char *mask, const char *fina) {
       if (*mask == *fina) return (filter(mask+1,fina+1)); else return (0);
 }
 
-#define ITMAX 200  // max # of items in each selector
+#define ITMAX 1000  // max # of items in each selector
 #define BHEIGHT 18 // button height
 #define ITDISP 20  // max of displayed items in selectors
 
@@ -362,8 +361,8 @@ class file_selection_box : public main_window {
 public: 
   FILE *fptr;
   char *fname;
-  char strings[10000]; // all names in one field
-  char *fitems[ITMAX],*ditems[ITMAX]; // max 200 entries into this field
+  char strings[40000]; // all names in one field
+  char *fitems[ITMAX],*ditems[ITMAX]; // max 1000 entries into this field
   char pwd[100]; // current directory path
   int fits, dits;
 private:
@@ -371,7 +370,7 @@ private:
   file_display *fdsp;
   selector *fsel,*dsel;
   button *okb, *abortb, *chb;
-
+public:   
   // searches pwd for matching files and displays them in fsel
   void make_select(char *mask) {
     DIR *dirp = opendir(".");
@@ -384,7 +383,7 @@ private:
       if ((dirfi && (dits < ITMAX)) || 
           (regfi && filter(mask,dp->d_name) && (fits < ITMAX))) {	 
 	int nl = strlen(dp->d_name) + 1; 
-	if ((nl + nstrp) >= 4000) error("too long name strings");
+	if ((nl + nstrp) >= 40000) error("too long name strings");
         char *strent = strings + nstrp; // entry point in strings
 	strcpy(strent,dp->d_name); 
       
@@ -401,15 +400,13 @@ private:
     XResizeWindow(display,Win,220,wh+110);
     dsel->resize(80,ddits*BHEIGHT + 6);
     fsel->resize(110,ffits*BHEIGHT + 6); 
-
     // printf("window %d %d\n",Win,wh+110);
   }
-   
   void ok() { 
     fptr = fopen(fname,"r");
     if (fptr) exit_flag = True; else printf("error opening file %s\n",fname);
   }
-
+  void ok_cb(char*) { ok(); } 
   void fsel_cb(char *val) { fname = val; fdsp->set_val(val);  }
   // callback for dsel enter 
   void dsel_cb(char *val) { 
@@ -427,30 +424,8 @@ private:
     chdir(val); 
     getcwd(pwd,100); fdsp->set_val(pwd);  
     make_select(mask_ed->value); 
- }
-public:
-  file_selection_box() : 
-    main_window("file selection box",220,250) {
-    
-    char empty = 0; fname = &empty; 
-    fdsp = new file_display(*this,200,20,10,10);
-    new text_win(*this,"selection mask",200,20,10,30);
-    mask_ed = new mask_edit(*this, (VPCP) &make_select, this,200,20,10,50);
-    // directory selector 
-    dsel = new selector(*this, ITDISP, (VPCP) &dsel_cb, (VPCP) &cd_cb,
-		       80,0,10,80,BHEIGHT);
-    // file selector
-    fsel = new selector(*this, ITDISP, (VPCP) &fsel_cb, (VPCP) &ok,
-		       110,100,100,80,BHEIGHT);
-    okb = new instance_button <file_selection_box> 
-                 (*this,"Ok",&file_selection_box::ok,this,50,20,0,0);
-    chb = new instance_button <file_selection_box> 
-                 (*this,"Mask",&file_selection_box::chmask,this,50,20,0,0);
-    abortb =
-       new variable_button <int> (*this,"Abort",&exit_flag,True,50,20,0,0);
+  }
 
-  }   
- 
   void resize(int w, int h) { 
     if (width == w && height == h) return; // only once
     // printf("resize %x ->  %d %d\n",Win,w,h); 
@@ -459,6 +434,8 @@ public:
     XMoveWindow(display,chb->Win,75,height-25);
     XMoveWindow(display,abortb->Win,140,height-25);
   } 
+
+  file_selection_box();
   ~file_selection_box() { }
 
   // calls main_loop, returns selected filename and opened file
@@ -474,6 +451,30 @@ public:
   }
 };
 
+static void fsb_make_select(file_selection_box *p, char* cp) { p->make_select(cp); }
+static void fsb_dsel_cb(file_selection_box *p, char* cp) { p->dsel_cb(cp); }
+static void fsb_fsel_cb(file_selection_box *p, char* cp) { p->fsel_cb(cp); }
+static void fsb_cd_cb(file_selection_box *p, char* cp) { p->cd_cb(cp); }
+static void fsb_ok(file_selection_box *p, char*) { p->ok(); }
+  
+file_selection_box::file_selection_box() : main_window("file selection box",220,250) {    
+  char empty = 0; fname = &empty; 
+  fdsp = new file_display(*this,200,20,10,10);
+  new text_win(*this,"selection mask",200,20,10,30);
+  mask_ed = new mask_edit(*this, (VPCP) &fsb_make_select, this,200,20,10,50);
+  // directory selector 
+  dsel = new selector(*this, ITDISP, (VPCP) &fsb_dsel_cb, (VPCP) &fsb_cd_cb,
+		      80,0,10,80,BHEIGHT);
+  // file selector
+  fsel = new selector(*this, ITDISP, (VPCP) &fsb_fsel_cb, (VPCP) &fsb_ok,
+		      110,100,100,80,BHEIGHT);
+  okb = new instance_button <file_selection_box> 
+    (*this,"Ok",&file_selection_box::ok,this,50,20,0,0);
+  chb = new instance_button <file_selection_box> 
+    (*this,"Mask",&file_selection_box::chmask,this,50,20,0,0);
+  abortb = new variable_button <int> (*this,"Abort",&exit_flag,True,50,20,0,0);
+}   
+
 // simple interface function
 // creates popup window and calls its main_loop 
 // the opened FILE* is returned and its filename (with strcpy)
