@@ -5,7 +5,7 @@
 
 double t[NZP2][NYP2][NXP2], tp[NZP2][NYP2][NXP2], dt[NZP2][NYP2][NXP2];
 
-double gamma;
+double gammad;
 double deltat = 10;
 double cdx = 0.001; // paramter == c*dx
 
@@ -20,7 +20,7 @@ void wave_init(int *tptr) {
   for (x=4; x<7; x++) for (y=4; y<7; y++) for (z=0; z<3; z++) 
    t[z][y][x] = 0.02; // initial impuls
   *tptr = 0;
-  gamma = sqr(cdx*deltat);
+  gammad = sqr(cdx*deltat);
   //  printf("init %x\n",&t);
 }
 
@@ -37,7 +37,7 @@ void wave_step(int *tptr) {
     if (z > 0)      dt2 -= t[z-1][y][x]; 
     if (z < NZP2-1) dt2 -= t[z+1][y][x];
        
-	t[z][y][x] =  2*tt - tp[z][y][x] - gamma*dt2;
+	t[z][y][x] =  2*tt - tp[z][y][x] - gammad*dt2;
     dt[z][y][x] = tt -  tp[z][y][x];
     tp[z][y][x] = tt;
   }
