@@ -5,8 +5,11 @@
 //     14478 Potsdam, Germany
 //     0331-863238
 
+#pragma implementation
+
 #include <assert.h>
 #include "window.h"
+
 #include "X11/Xutil.h"
 #include "X11/cursorfont.h"
 
@@ -235,10 +238,9 @@ window::window(window & parent, int w, int h, int x, int y, int bw) {
   children = NULL; // own children List = empty
 
   parent.add_child(this,x,y);
-  help = NULL;
+  dhlp = 0;
   type = SimpleType;
-  Win = XCreateSimpleWindow(display, parent.Win, x,y, width, height,
-			    border_width,black,white); 
+  Win = XCreateSimpleWindow(display, parent.Win, x,y, width, height, border_width,black,white); 
  
   if (debug_create) printf(" Window %lx %d %d \n",Win,width, height);
   
@@ -357,16 +359,19 @@ void window::clear() {
   XFillRectangle(display, Win, gc_clear, 0,0,width,height); 
 }
 
-void window::DrawString(int x, int y, char * string) { 
-  XDrawString(display,Win,text_gc,x,y, string, strlen(string)); 
+int window::DrawString(int x, int y, char * string) { 
+  int n = strlen(string);
+  XDrawString(display,Win,text_gc,x,y, string, n);
+  return n;
 }
 
-void window::printw(int x, int y, char* fmt, ...) {
+int window::printw(int x, int y, char* fmt, ...) {
   va_list args; va_start(args,fmt); 
   char str[1000]; 
   vsprintf(str, fmt, args);
-  DrawString(x,y,str);
+  int n = DrawString(x,y,str);
   va_end(args);
+  return n;
 }
 
 void window::PlaceText(char * string, int x, int y, XFontStruct * fn) { 
@@ -379,18 +384,13 @@ void window::PlaceText(char * string, int x, int y, XFontStruct * fn) {
   XSetFont(display, text_gc, fixed_fn->fid);
 }
 
-void window::line(int x0, int y0, int x1, int y1) {
+void window::line(int x0, int y0, int x1, int y1) { 
   XDrawLine(display, Win, gc_copy, x0, y0, x1, y1); 
 }
 
-void window::DrawPoint(int x, int y) {
-  XDrawPoint(display, Win, gc_copy, x, y); 
-}
+void window::DrawPoint(int x, int y) { XDrawPoint(display, Win, gc_copy, x, y); }
 
-void window::BPress_3_CB(XButtonEvent ev) { 
-  if (help) help->do_popup(ev.x_root,ev.y_root + 20);
-  //  popup below cursor pos. else use:   help->RealizeChildren();
-}
+#include "helper.c"
 
 void window::Expose_CB(XExposeEvent ev) { 
   if (CB_info) 
@@ -419,27 +419,14 @@ void window::Expose_CB(XExposeEvent ev) {
    MappingNotify	34  LASTEvent		35
 */
 
-// the main callback function
-pulldown_button * active_button = NULL;
 
-// global list : used to destroy an pulldown window 
-// with the next button release 
-// in the CallBack function; not very nice though
-window * pulldown_mapped = NULL; 
-
-void window::CallBack(XEvent &event) 
-{ if (CB_info) printf("Event %s (%d) in Win %lx\n",event_names[event.type],
-		       event.type,event.xany.window);
+// the main callback function manages call of proper event handler
+void window::CallBack(XEvent &event) { 
+  if (CB_info) printf("Event %s (%d) in Win %lx\n",event_names[event.type],
+		      event.type,event.xany.window);
   if (hidden) return;
-  if ((event.type == ButtonRelease))
-    { if (pulldown_mapped) // hat ein pulldown menu gemapped -> unmap it
-	{ pulldown_mapped->Unmap(); pulldown_mapped = NULL; }  
-      
-      if (active_button) // restore invoker button
-	{ active_button->default_frame(); active_button = NULL;}
-    }
-  switch (event.type) 
-    { 
+  
+  switch (event.type) { 
     case ButtonPress:    
         BPress_CB(event.xbutton); // allgemeiner BPress-handler, zB. buttons   
 	switch (event.xbutton.button) {
@@ -448,7 +435,7 @@ void window::CallBack(XEvent &event)
 	}
         break;
     case ButtonRelease:
-        BRelease_CB(event.xbutton);  break;
+        BRelease_CB(event.xbutton); break;
     case EnterNotify: 
 	Enter_CB(event.xcrossing); break;
     case LeaveNotify:
@@ -464,13 +451,7 @@ void window::CallBack(XEvent &event)
     case ClientMessage:
 	ClientMsg_CB(event.xclient); break;
     default: break;
-    }; 
-}
-
-void window::add_help(char * WMName,char * text[]) { 
-  int lines, cols;
-  compute_text_size(text,cols,lines);
-  help = new text_popup(WMName,6*cols + 10, 20*lines + 30,text);
+   }; 
 }
 
 void window::WatchCursor() { // set main window to watch
@@ -525,22 +506,16 @@ main_window::main_window(char * WMName, int w, int h,int fix_pos,int x,int y)
   // printf("1.main %s %d\n", WMName, fix_pos);
   XDefineCursor(display, Win, main_cursor);
   set_icon(icon_bits,icon_width,icon_height);
- 
-  switch (fix_pos) {       
-  case 2: 
-    x = (DisplayWidth(display,screen) - w)/2;
-    y = (DisplayHeight(display,screen)- h)/2;  // centre window
+  if (fix_pos > 0) { 
+    XSizeHints sz_hints; 
+    if (fix_pos == 2) {       
+      x = (DisplayWidth(display,screen) - w)/2;
+      y = (DisplayHeight(display,screen)- h)/2;  // centre window
+    }
     XMoveWindow(display,Win, x, y); // XSetWMNormalHints doesnt work here ?!
-    // thats why the explicit move is inserted
-    // no break;  
-  case 1:  // x,y have been given explicitely
-    XSizeHints *sz_hints = XAllocSizeHints(); 
-    sz_hints->flags = PPosition;
-    sz_hints->x = x;
-    sz_hints->y = y;
-    XSetWMNormalHints(display,Win,sz_hints);
-    XFree(sz_hints);      
-    break;
+    // the x,y-slots of sz_hints are ignored, only move works ! 
+    sz_hints.flags = PPosition | USPosition;;
+    XSetWMNormalHints(display,Win,&sz_hints);
   }
   // this protocoll is used from the window manager (mwm) when the 
   // close button is pushed (f.kill). If not set the whole application
@@ -567,8 +542,8 @@ main_window::~main_window() {
 // 1. make it small before moving it, 2. realize, 3. move to x,y-position
 // not quite convinient
 // this version -> does not work !
-void main_window::do_popup(int x , int y)
-{ XMoveWindow(display, Win, x,y); // -> does not work here
+void main_window::do_popup(int x , int y) { 
+  XMoveWindow(display, Win, x,y); // -> does not work here
   RealizeChildren(); 
   // XMoveResizeWindow(display, Win, x, y, width, height); // -> too slow 
 }
@@ -585,12 +560,13 @@ void main_window::Configure_CB(XConfigureEvent ev) {
 // corresponding derived class from window
 
 void handle_event(XEvent &event) {
+  if (event.type != MotionNotify) clear_help(); // if there is any open help window -> close it
+  // but not for PointerMotion Events (scrollbar helps !)
   unsigned wid = event.xany.window;
-
   xlist *xw = xhatab->xhget(wid);
-  if (xw) xw->tw->CallBack(event); 
+  if (xw) xw->tw->CallBack(event);
   // else : the window is deleted, but some events can occur nevertheless 
-  //  eg. LeaveNotify-Events (type = 8) 
+  // eg. LeaveNotify-Events (type = 8) 
 }
 
 // for event loop in polling_mode
@@ -652,8 +628,11 @@ void pixmap_window::Map() {
   XMapWindow(display, Win); 
 }
 
-void pixmap_window::DrawString(int x, int y, char * string) {
-  XDrawString(display,pix,gc_copy,x,y, string, strlen(string)); }
+int pixmap_window::DrawString(int x, int y, char * string) {
+  int n = strlen(string);
+  XDrawString(display,pix,gc_copy,x,y, string, n); 
+  return n;
+}
 
 void pixmap_window::line(int x0, int y0, int x1, int y1) {
   XDrawLine(display, pix, gc_copy, x0, y0, x1, y1); }
@@ -721,6 +700,7 @@ plate::plate(window & parent, int w, int h, int x, int y,int mode3d)
 : window(parent, w, h, x, y, 0), mode3d(mode3d) { 
   selection_mask |= EnterWindowMask | LeaveWindowMask; 
 }
+
 void plate::redraw() {
   XFillRectangle(display,Win,button_fg_gc,0,0,width,height); 
   default_frame(); // virtual function !
@@ -731,22 +711,27 @@ void plate::redraw() {
 
 void button::init_button(window *parent) {  
   in_pulldown = (parent->type == PulldownType);
+  enabled = true; // by default
   selection_mask |= ButtonPressMask | ButtonReleaseMask;
 } 
+
 void button::redraw() {
   plate::redraw(); 
-  PlaceText(Name); 
+  PlaceText(Name);
+  if (! enabled) for (int y=3; y < height-3; y +=3) // some lines with button color : dither
+    XDrawLine(display,Win,button_fg_gc,3,y,width-3,y); 
 }
 
 button::~button() {
   if (debug_delete) printf("button '%s' -> ",Name); 
 }
 
-void button::add_help(char **help_text) {
-    char * WMName = new char[strlen(Name) + 12]; 
-    sprintf(WMName,"Help : '%s'",Name);
-    window::add_help(WMName,help_text); }
-
+void button::BRelease_CB(XButtonEvent ev) { 
+  if (ev.state & Button1Mask) { 
+    if (enabled) BRelease_1_action(); 
+    default_frame(); 
+  }
+}
 // ##################      popup_button        #########################
 // realize the popup menu when BPress (make it visibel) 
 // if pressed again : make it unvisible 
@@ -814,16 +799,11 @@ void switch_button::switch_it() {
 // a window child from root_window, not yet visible, not managed from WM,
 // position is determined from the button at popup time
 
-// special cursor for pulldown menus 
-Cursor pulldown_cursor = XCreateFontCursor(display,XC_right_ptr);
-
-pulldown_window::pulldown_window (int w, int h) : main_window("",w,h) {
+pulldown_window::pulldown_window (int w, int h, int nb, button **rbp) : 
+main_window("",w,h), nb(nb), rbp(rbp){
   XSetWindowAttributes attr; 
   attr.override_redirect = TRUE; attr.save_under = TRUE;
-  attr.cursor = pulldown_cursor; 
-  XChangeWindowAttributes(display, Win, 
-			  CWOverrideRedirect | CWCursor | CWSaveUnder, 
-			  &attr);
+  XChangeWindowAttributes(display, Win, CWOverrideRedirect | CWSaveUnder, &attr);
   type = PulldownType;
   rold = NULL;
 }
@@ -833,17 +813,49 @@ void pulldown_window::toggle(radio_button *rb) {
   // printf("%p %p %p\n",this,rold,rb);
   rold = rb; rb->status = True;
 }
+
+// maps pulldown window and handles X-pointer selection (only for local buttons!!)
+void pulldown_window::handler(int x, int y) {  
+  XMoveWindow(display, Win, x, y);
+ 
+  RealizeChildren();
+  XRaiseWindow(display,Win); 
+
+  XUngrabPointer(display,CurrentTime); // to enable the pulldown menu
+  // for (int i=0; i<nb; i++) printf("  %lx",rbp[i]->Win);
+  // printf("\n");
+
+  while (1) { 
+    XEvent event;   
+    XNextEvent(display, &event);
+    
+    unsigned wid = event.xany.window;
+    for (int i=0; i<nb; i++) 
+      if (wid == rbp[i]->Win) {
+	xlist *xw = xhatab->xhget(wid);
+	if (xw) xw->tw->CallBack(event); 
+      }
+    if (event.type == ButtonRelease) break;
+  }
+  Unmap();
+
+}
 // ##################     pulldown_button class    ########################
 // map the window (menu) on root when button is activated (BPress)
 // the window is mapped to absolute co-ordinates !
 
 pulldown_button::pulldown_button (window & parent, pulldown_window * menu, 
 				  char * Name, int w, int h, int x, int y) 
-: button(parent, Name, w, h, x, y) { pulldown_menu = menu; xright = 12;}
+: button(parent, Name, w, h, x, y) { 
+  pulldown_menu = menu; xright = 12; 
+}
 
 pulldown_button::pulldown_button(menu_bar & parent, pulldown_window * menu, 
 				 char * Name) 
-: button(parent, ext_string(Name)) { pulldown_menu = menu; xright = 12; }
+: button(parent, ext_string(Name)) { 
+  pulldown_menu = menu; xright = 12; 
+
+}
 
 pulldown_button::pulldown_button (window & parent, pulldown_window * menu, 
 				  char * Name, char ** help_text,
@@ -856,28 +868,17 @@ void pulldown_button::picture() {
    tri3d_s(Win,up3d, width-xright,offs,8,8);
 }
 
-void pulldown_button::BPress_1_CB(XButtonEvent ev) 
-{ 
-  unsigned x, y; // *absolut*  position of pulldown menu
+// special cursor for pulldown menus 
+Cursor pulldown_cursor = XCreateFontCursor(display,XC_right_ptr);
+
+void pulldown_button::BPress_1_CB(XButtonEvent ev) { 
+  int x, y; // *absolut*  position of pulldown menu
   // determine the absolute coord of mouse pointer
   x = ev.x_root - ev.x; // + (width-pulldown_menu->width)/2; horizontal centr.
   y = ev.y_root - ev.y + height + 1;
 
-  XMoveWindow(display, pulldown_menu->Win, x, y);
- 
-  pulldown_menu->RealizeChildren();
-  XRaiseWindow(display,pulldown_menu->Win); 
-
-  XGrabPointer(display,pulldown_menu->Win,TRUE,
-	       Button1MotionMask | PointerMotionMask |
-	       EnterWindowMask | LeaveWindowMask 
-	       , GrabModeAsync, GrabModeAsync, None, None, CurrentTime);
-  pulldown_mapped = pulldown_menu; 
-  active_button = this;
-  // with this trick the next BRelease causes the unmapping of the pulldown
-  // and restores the default_frame (this)
-  // not to solve with class method since BRelease is invoked for another win
-  // should be replaced by push/pop method, to allow multiple popups
+  pulldown_menu->handler(x,y);
+  default_frame();
 } 
 
 // if w == 0 parent must be menu_bar -> use autoplacement
@@ -891,37 +892,33 @@ pulldown_button* make_radio_menu(window &parent, char *Name, char **list,
   compute_text_size(list,wm,nbuttons);
   wm = 40 >? wm * 6 + 18;   //  6 = width of char
   hm = nbuttons * 20;       // 20 = height of button
-  pulldown_window * pd_wi = new pulldown_window(wm,hm);
-  pulldown_button * pd_bt;
-  if (w > 0) pd_bt = new pulldown_button(parent,pd_wi,Name,w,h,x,y); else
-     pd_bt = new pulldown_button((menu_bar&) parent, pd_wi, Name);
-  int yp = 0, ind = 0;  
-  while (*list) { 
+  radio_button **rbp = new (radio_button*) [nbuttons];
+  pulldown_window *pd_wi = new pulldown_window(wm,hm,nbuttons, (button**) rbp);
+  pulldown_button *pd_bt = (w > 0) ? new pulldown_button(parent,pd_wi,Name,w,h,x,y) 
+    : new pulldown_button((menu_bar&) parent, pd_wi, Name);
+
+  for (int yp = 0, ind = 0; ind < nbuttons; ind++, yp += 20) { 
     value = *list++;
-    radio_button *rb =
+    radio_button *rb = rbp[ind] = 
       new radio_button(*pd_wi,Name,wm,20,0,yp,value, action_win);
-    Bool status = (ind++ == inival);
+    Bool status = (ind == inival);
     rb->status = status; // the first button is set True, others False
     if (status) pd_wi->toggle(rb);
-    yp += 20;
   };
   return pd_bt;
 }
 
-pulldown_button* make_radio_menu(menu_bar &parent, char *Name,
-				  char **blist,  window * action_win,
-				  int inival) {
-  pulldown_button* r_menu = make_radio_menu(parent, Name, blist,
-					    action_win, 0, 0, 0, 0, inival );
+pulldown_button* make_radio_menu(menu_bar &parent, char *Name, char **blist, window* action_win,
+				 int inival) {
+  pulldown_button* r_menu = make_radio_menu(parent, Name, blist, action_win, 0, 0, 0, 0, inival );
   return r_menu;
 }
 
 // analogous : with help popup
 pulldown_button* make_radio_menu(window &parent, char *Name, char **list,
-				  char ** help_text, window * action_win,
-				  int w, int h, int x, int y, int inival) {
-  pulldown_button* r_menu = make_radio_menu(parent, Name, list,
-					    action_win, w, h, x, y, inival );
+				 char** help_text, window* action_win,
+				 int w, int h, int x, int y, int inival) {
+  pulldown_button* r_menu = make_radio_menu(parent, Name, list, action_win, w, h, x, y, inival );
   r_menu->add_help(help_text);
   return r_menu;
 }
@@ -939,14 +936,15 @@ pulldown_button * make_pulldown_menu(window &parent, char *Name,
   };
   wm = wm * 6 + 10 ;            //  6 = length of char
   hm = nbuttons * 20;           // 20 = height of button
-  pulldown_window * pd = new pulldown_window(wm, hm);
+  button **cbp = new (button*)[nbuttons];
+  pulldown_window * pd = new pulldown_window(wm, hm,nbuttons,cbp);
   pulldown_button * pb;
   if (w > 0) pb = new pulldown_button(parent,pd,Name,w,h,x,y); else
      pb = new pulldown_button((menu_bar&) parent, pd, Name); 
 
   int yp = 0; 
   for (i=0; i < nbuttons; i++) { 
-    new callback_button(*pd,list[i].bname,list[i].cb,wm,20,0, yp);
+    cbp[i] = new callback_button(*pd,list[i].bname,list[i].cb,wm,20,0, yp);
     yp += 20;
   }
   return pb;
@@ -1146,14 +1144,16 @@ scrollbar::scrollbar(window &parent, void (*inf)(void*), void * to_inf,
        inffn.vptr = inf;
   }
     
-void scrollbar::callbck_val(float x) {   
+void scrollbar::display_val(float x) {   
   // quick clearing of the old display : overwrite with  background gc
   disp_win->set_text_mode(0); disp_win->draw_val();
   setval(x); // compute new display
-  disp_win->set_text_mode(1); disp_win->draw_val();
-  // reset default gc !!! , new value
-  if (inffn.value) 
-    { if (to_inform) (*inffn.vptr)(to_inform); else (*inffn.empty)(); }
+  disp_win->set_text_mode(1); disp_win->draw_val(); // reset default gc !!! , new value
+}
+
+void scrollbar::callbck_val(float x) {  
+  display_val(x);
+  if (inffn.value) { if (to_inform) (*inffn.vptr)(to_inform); else (*inffn.empty)(); }
 }
 
 void scrollbar::resize(int w, int h) {
@@ -1201,25 +1201,28 @@ void tick_scrollbar::adapt(int maxp, int xstart) {
   }
 }
 
+// some aux fns
+static void play_updated(play_scrollbar *psc) { psc->updated(); }
+static void play_switch(play_scrollbar *psc) { psc->switch_play(); }
+
 // **** class play_scrollbar ***
 // a tick_scrollbar, that adds playback- and stepping buttons left and right  
 play_scrollbar::play_scrollbar(window &parent, int w, int x, int y, 
 			       int n_ticks, stepper *stp, int minp, int maxp, 
 			       int xstart) 
-: stp(stp),
-  tick_scrollbar(parent, (VVP) &play_scrollbar::updated, this, w-2*(15+20), 
-		 x+50, y, n_ticks, minp, maxp, xstart) {   
-    new instance_button <play_scrollbar> // left
-      (parent,"<", &play_scrollbar::step_bck, this, 15,20, x, y); 
+: tick_scrollbar(parent, (VVP) &play_updated, (void*) this, w-2*(15+20), 
+		 x+50, y, n_ticks, minp, maxp, xstart), stp(stp) {   
+  new instance_button <play_scrollbar> // left
+    (parent,"<", &play_scrollbar::step_bck, this, 15,20, x, y); 
     
-    new instance_button <play_scrollbar> // left +15
-      (parent,">", &play_scrollbar::step_fwd, this, 15, 20, x+15, y); 
-    
-    new switch_button(parent,">>","||",&play_mode, // left + 30
-		      (VVP) &play_scrollbar::switch_play,this, 20, 20, x+30,y);
-    new instance_button <play_scrollbar>  // right - 20
-      (parent,"<<", &play_scrollbar::reset, this, 20, 20, x+w-20, y);
-  }
+  new instance_button <play_scrollbar> // left +15
+    (parent,">", &play_scrollbar::step_fwd, this, 15, 20, x+15, y); 
+  
+  new switch_button(parent,">>","||",&play_mode, // left + 30
+		    (VVP) &play_switch, (void*) this, 20, 20, x+30,y);
+  new instance_button <play_scrollbar>  // right - 20
+    (parent,"<<", &play_scrollbar::reset, this, 20, 20, x+w-20, y);
+}
 
 // ***************************************
 // class "edit_window" for editing of strings, max 80 chars 
@@ -1406,6 +1409,10 @@ void scrolled_window::cbhook(twodim_input *ts) {
   XMoveWindow(display,virt_win->Win,xp,yp);
 }
 
+static void scw_cbhook(scrolled_window* sc, twodim_input *td) {
+  sc->cbhook(td);
+}
+
 void scrolled_window::resize(int w, int h) {
   // if (w == width && h == height) return;
   width = w; height = h;
@@ -1421,13 +1428,13 @@ void scrolled_window::resize(int w, int h) {
   if (vert_scr) {
     int sh = (h-4) * h/hvirt; if (sh < 4) sh = 4; // else too small
     if (vs == 0) // create new 
-      vs = new vertical_shifter(*this,sz,h,w,0,sh, (CBHOOK) &cbhook, this);
+      vs = new vertical_shifter(*this,sz,h,w,0,sh, (CBHOOK) &scw_cbhook, this);
     else vs->configure(sz,h,0,sh,w,0); 
   } else if (vs) XUnmapWindow(display,vs->Win);
   if (horz_scr) {
     int sw = (w-4) * w/wvirt; if (sw < 4) sw = 4; // else too small
     if (hs == 0) // create new 
-      hs = new horizontal_shifter(*this,w,sz,0,h,sw, (CBHOOK) &cbhook,this);
+      hs = new horizontal_shifter(*this,w,sz,0,h,sw, (CBHOOK) &scw_cbhook,this);
     else hs->configure(w,sz,sw,0,0,h); 
   } else if (hs) XUnmapWindow(display,hs->Win);
   
