@@ -7,6 +7,7 @@
 
 #if !defined WINDOW_H
 #define WINDOW_H
+#pragma interface 
 
 #include <X11/Xlib.h>
 // #include "X11/Xutil.h"
@@ -68,19 +69,17 @@ struct win_list { // struct to manage a child list of any window
   window *child; 
   int x,y;
   struct win_list *next;
-  win_list(window *ch, int x, int y, win_list *rest) : child(ch), x(x), y(y) 
-    { next = rest; }
+  win_list(window *ch, int x, int y, win_list *rest) : child(ch), x(x), y(y) { next = rest; }
 };
 
+struct dhelper; // for dynamic help on EnterCB
 class main_window;
-class text_popup;
 
 class window {
-
 protected:
   int border_width;
   GC gc,text_gc;
-  text_popup * help; // ggf. help popup
+  dhelper *dhlp;     // on EnterCB
   window *parentw;   // only for destructors
   long selection_mask; // default mask for SelectInput
   Bool hidden;    // if true not realize the window 
@@ -90,12 +89,10 @@ public:
   int CB_info;   // toggle debug infos on callbacks in this Window
   Window Win;
   int type;      // to distinguish simple <-> pulldown
-
-  win_list * children;        // chained List of children
+  win_list *children;        // chained List of children
  
-  window(char * DISP = NULL); // constructor for root window
-  window(window & parent,
-	 int w = 0, int h = 0, int x = 0, int y = 0, int bw = 1);
+  window(char *DISP = NULL); // constructor for root window
+  window(window & parent, int w = 0, int h = 0, int x = 0, int y = 0, int bw = 1);
   // to update children list and fit the parent geometry :
   virtual void add_child(window *ch, int x, int y);
   
@@ -116,18 +113,18 @@ public:
   virtual void clear();
   virtual int eff_width() { return width; } // effective breadth minus pictures
 
-  virtual void DrawString(int x, int y, char* string);
+  virtual int DrawString(int x, int y, char* string); // returns string length
 
-  void printw(int x, int y, char* fmt, ...); // called like printf, max 1000 !
+  int printw(int x, int y, char* fmt, ...); // called like printf, max 1000 !
 
   // place a string with font (fixed_fn) at x,y (0 = centered)
-  void PlaceText(char * string,int x = 0, int y = 0,
-		   XFontStruct * fn = fixed_fn);
+  void PlaceText(char* string,int x = 0, int y = 0, XFontStruct * fn = fixed_fn);
   virtual void line(int x0, int y0, int x1, int y1);
   virtual void DrawPoint(int x, int y);
+  // new help functions : popup on enter-cb
+  void add_help(char* help[]);              // given as vector
+  void add_help(char* arg1,...);            // as arg list
 
-  void add_help(char * WMName,char * text[]);
-  
   virtual void action(char*, char*) {} // action for windows, that shall
   // be managed from radio_button - BPress-CBs
 
@@ -138,9 +135,9 @@ public:
 protected:  
   virtual void BPress_CB(XButtonEvent) {} 
   virtual void BPress_1_CB(XButtonEvent) {}
-  virtual void BPress_3_CB(XButtonEvent); // default : popup help
+  virtual void BPress_3_CB(XButtonEvent) {} // obsolete (was popup help)
   virtual void BRelease_CB(XButtonEvent) {} 
-  virtual void Enter_CB(XCrossingEvent) {}
+  virtual void Enter_CB(XCrossingEvent); 
   virtual void Leave_CB(XCrossingEvent) {}
   virtual void Motion_CB(XMotionEvent) {} // pointer movements
   virtual void Expose_CB(XExposeEvent);
@@ -163,7 +160,7 @@ protected:
   char *name;
 public: 
   int exit_flag; // used in main_loop to break it
-  main_window(char *Name, int w, int h, int fix_pos = 0, int x = 0, int y = 0);
+  main_window(char *Name, int w, int h, int fix_pos = 2, int x = 0, int y = 0);
   ~main_window();
   void do_popup(int x, int y); // Realize at absolute x,y-coords
   virtual void Configure_CB(XConfigureEvent ev);
@@ -185,7 +182,7 @@ public:
   virtual ~pixmap_window();
   virtual void clear();
   virtual void Map(); // copy pix -> Win
-  virtual void DrawString(int x, int y, char * string);
+  virtual int DrawString(int x, int y, char * string);
   virtual void line(int x0, int y0, int x1, int y1);
   virtual void DrawPoint(int x, int y);
   virtual void Expose_CB(XExposeEvent ev); 
@@ -219,8 +216,7 @@ class horizontal_container {
 public:
   int xp, yp; // place position for next child, is updated from children
   int getw(int w) { return ((w > min_w) ? ((w > max_w) ? max_w : w) : min_w) ;}
-  horizontal_container(int minw, int maxw) : min_w(minw), max_w(maxw) 
-  { xp = yp = 0; }
+  horizontal_container(int minw, int maxw) : min_w(minw), max_w(maxw) { xp = yp = 0; }
 };
 
 class menu_bar : public window, public horizontal_container {
@@ -262,40 +258,34 @@ class button : public plate {
 protected:
   // the breadth of strings for  menu_bar (for auto-placement in container)
   int Nwidth(char *str) { return 6*strlen(str) + 6; }
+  bool enabled; // active or not
 public:
   char * Name;
-  void add_help(char **help_text); // replaces window::add_help
   void init_button(window *parent);
-  button (window & parent, char * Name, int w, int h, 
-	  int x = 0, int y = 0) :
+  button (window & parent, char * Name, int w, int h, int x = 0, int y = 0) :
   plate (parent, w, h, x, y), Name(Name) { init_button(&parent); }
 
-  button (window & parent, char * Name, char ** help_text, int w, int h,
-	  int x = 0, int y = 0) : plate (parent, w, h, x, y), Name(Name) 
-    { add_help(help_text); init_button(&parent); }
+  button (window & parent, char* Name, char** /*help_text*/, int w, int h, int x = 0, int y = 0) : 
+  plate (parent, w, h, x, y), Name(Name) { init_button(&parent); }
   
   // autosizing buttons
   button (menu_bar & parent, char * Name) :
-    Name(Name), plate (parent, parent.getw(Nwidth(Name)),  
-                       parent.height, parent.xp, parent.yp) 
-    { init_button(&parent); }
+   plate (parent, parent.getw(Nwidth(Name)), parent.height, parent.xp, parent.yp), 
+   Name(Name) { init_button(&parent); }
   virtual ~button();
   virtual void default_frame() { frame3d(in_pulldown ? flat3d : up3d);  }
   void enter_frame() { frame3d(in_pulldown ? up3d : flat3d); }
   virtual void redraw(); 
+  void disable() { enabled = false; redraw(); }
+  void enable() { enabled = true; redraw(); }
 protected: 
-  virtual void Enter_CB(XCrossingEvent) { enter_frame(); }
-  virtual void Leave_CB(XCrossingEvent) { default_frame(); }
-  virtual void BPress_CB(XButtonEvent) { frame3d(down3d); }
+  virtual void Enter_CB(XCrossingEvent ev) { window::Enter_CB(ev); if (enabled) enter_frame(); }
+  virtual void Leave_CB(XCrossingEvent ev) { window::Leave_CB(ev); default_frame(); }
+  virtual void BPress_CB(XButtonEvent) { if (enabled) frame3d(down3d); }
 
-  virtual void BPress_1_CB(XButtonEvent ) { 
-    // printf("button press '%s' %d \n", Name, ev.button); 
-  }
+  virtual void BPress_1_CB(XButtonEvent ) {}
   virtual void BRelease_1_action() {} // hook for callbacks with Button1
-  virtual void BRelease_CB(XButtonEvent ev) { 
-    if (ev.state & Button1Mask) { BRelease_1_action(); default_frame(); }
-  }
-
+  virtual void BRelease_CB(XButtonEvent ev);
 };
 
 //                 ##### delete_button #####
@@ -343,8 +333,8 @@ protected:
 // the popup window is created in make_popup from the text array
 class help_button : public popup_button {
 public: 
-  help_button(window & parent, int x, int y, char * text[]) :
-    popup_button(parent, NULL, "help", 60, 20, x, y) { make_popup(text); }
+  help_button(window & parent, int x, int y, char * text[], int w = 60, int h = 20) :
+    popup_button(parent, NULL, "help", w, h , x, y) { make_popup(text); }
 
   help_button(menu_bar & parent, char * Name, char * text[]) :
     popup_button(parent,NULL, Name) { make_popup(text); }
@@ -440,7 +430,6 @@ typedef void (*VVP)(void *); // typecast for callbacks with void* argument
 extern void switch_dummy(void *); // default dummy fn : does nothing
 
 // ****** class switch_button *****
-
 // a button with 2 states of display, which switch on click
 // the 2. string should be <= the first (= initial)
 class switch_button : public button {
@@ -451,19 +440,15 @@ class switch_button : public button {
 public:  
   switch_button (menu_bar & parent, char *Name1, char *Name2, Bool *toggle,
 		 VVP inf = switch_dummy , void * toinf = NULL);
-
   switch_button (window & parent, char *Name1, char *Name2, Bool *toggle,
 		 VVP inf, void * toinf, int w, int h, int x, int y);
-
   void switch_it();
 protected:
   virtual void BPress_1_CB(XButtonEvent) { } // do nothing
   virtual void BRelease_1_action() { switch_it(); }
 };
 
-
 //     ****** class toggle_button : with display of state ****
-
 class toggle_button : public button {
   int *vptr; // the pointer to  toggle-value
   int xright; // place for picture on the right side
@@ -498,12 +483,15 @@ public:
 // pulldown window : a window child from root_window, not yet visible, 
 // not managed from WM, position is determined from the button at popup time
 class radio_button;
+
 class pulldown_window : public main_window {
   radio_button *rold; // the last selected radio_button
 public:
-  pulldown_window (int w, int h);
-  inline void Release_CB(XButtonEvent) { Unmap();}
+  int nb; // number of buttons
+  button **rbp; // vektor of (radio)buttons
+  pulldown_window (int w, int h, int nb, button **rbp);
   void toggle(radio_button *rb);
+  void handler(int x, int y); // to handle X-pointer
 };
 
 //     **** "pulldown_button" class *****
@@ -522,7 +510,8 @@ public:
 		   char ** help_text, int w, int h, int x, int y);
   void picture();
 protected:
-  virtual void BPress_1_CB(XButtonEvent ev);   
+  virtual void BPress_1_CB(XButtonEvent ev);  
+  virtual void BRelease_CB(XButtonEvent) { pulldown_menu->Unmap(); }
   virtual void Leave_CB(XCrossingEvent ev) { // if B1 was not pressed
     if (!(ev.state & Button1Mask)) default_frame(); }
   virtual void redraw() { button::redraw(); picture(); }
@@ -801,12 +790,15 @@ public:
 	    float minp=0, float maxp=0, float xstart=0, char *format= "%.0f");
 
   // explicite setting of slider and display 
-  void change(float x) { move( val_to_pix(x) ); callbck_val(x);} 
+  void change(float x) { move( val_to_pix(x) ); callbck_val(x);}   
+  // explicite setting of slider and display (without callback !) 
+  void set(float x) { move( val_to_pix(x) ); display_val(x); } 
 protected:
   virtual void callbck(int pix) {  // called from move_cb (mouse events)
     callbck_val( pix_to_val(pix) );
   }
-  virtual void callbck_val(float x);    // called from change
+  void display_val(float x);          // set and display new x
+  virtual void callbck_val(float x);  // called from change : disp_val + callback
   virtual void resize(int, int);
 };
 
@@ -828,6 +820,8 @@ public:
   void adapt(int maxp, int xstart);
 };
   
+class play_scrollbar;
+
 // container class for interaction between play_scrollbar and 
 // a lattice drawer class 
 // methods must be overwritten from derived classes,
@@ -835,7 +829,8 @@ public:
 class stepper {
 public:
   virtual void seek_step(float v) = 0; // seek and perform actual drawing
-  virtual Bool switch_play_mode(VVP, void*) = 0; // returns actual mode
+  virtual Bool switch_play_mode(void (play_scrollbar::*)(), play_scrollbar*) = 0; 
+  // returns actual mode
 };
 
 // **** class play_scrollbar ***
@@ -843,18 +838,15 @@ public:
 class play_scrollbar : public tick_scrollbar {
   stepper *stp;
   Bool play_mode;
-  // called when setting the slider, or indirectly from step_xxx
-  void updated() { stp->seek_step(value); }
-
   // button callback functions :
+public:
   void step_fwd() { change(value+1); } 
   void step_bck() { change(value-1); } 
   void reset() { change(min); }
+  // called when setting the slider, or indirectly from step_xxx
+  void updated() { stp->seek_step(value); }
+  void switch_play() { play_mode = stp->switch_play_mode(&step_fwd, this); }
 
-  void switch_play() {
-    play_mode = stp->switch_play_mode((VVP)&play_scrollbar::step_fwd,this);
-  }
-public:
   play_scrollbar(window &parent, int w, int x, int y, int n_ticks, stepper *stp,
 		 int minp = 0, int maxp = 0, int xstart = 0);
 };
@@ -969,9 +961,9 @@ public:
 class scrolled_window : public window {
   int xp, yp; // actual position
   int hvis,wvis; // visible window size
+public:
   // the hook called from shifters : shift drawing_area
   void cbhook(twodim_input *ts); 
-public:
   twodim_input *vs, *hs;
   window *clip; // the clipped region
   window *virt_win; // the virtual window
